@@ -2,7 +2,7 @@ package DateTime::Format::Flexible;
 use strict;
 use warnings;
 
-our $VERSION = '0.20';
+our $VERSION = '0.23';
 
 use base 'DateTime::Format::Builder';
 
@@ -24,6 +24,7 @@ my $AMPM = qr{(a\.?m?|p\.?m?)\.?}i;
 
 my $MMDDYYYY = qr{(\d{1,2})$DELIM(\d{1,2})$DELIM(\d{1,4})};
 my $YYYYMMDD = qr{(\d{4})$DELIM(\d{1,2})$DELIM(\d{1,2})};
+my $MMYY = qr{(\d{1,2})${DELIM}(\d{1,2})}; # YEAR must be > 31 unless MMYY
 my $MMDD = qr{(\d{1,2})$DELIM(\d{1,2})};
 my $XMMXDD = qr{X(\d{1,2})X${DELIM}?(\d{1,2})};
 my $DDXMMX = qr{(\d{1,2})${DELIM}?X(\d{1,2})X};
@@ -34,10 +35,12 @@ my $XMMXDDYYYY = qr{X(\d{1,2})X${DELIM}?(\d{1,2})${DELIM}?(\d{1,4})};
 
 my $HMSMD = [ qw( hour minute second month day ) ];
 my $HMSMDY = [ qw( hour minute second month day year ) ];
+my $HMSNSMDY = [ qw( hour minute second nanosecond month day year ) ];
 my $HMSDM = [ qw( hour minute second day month ) ];
 my $HMMDY = [ qw( hour minute month day year ) ];
 my $HMMD = [ qw( hour minute month day ) ];
 my $HMAPMMDD = [ qw( hour minute ampm month day ) ];
+my $HMAPMMDDYYYY = [ qw( hour minute ampm month day year ) ];
 my $DM = [ qw( day month ) ];
 my $DMY = [ qw( day month year ) ];
 my $DMHM = [ qw( day month hour minute ) ];
@@ -152,9 +155,27 @@ my $formats =
    postprocess => \&_fix_ampm },
 
  ########################################################
+ ##### Month/Year
+ ##### year must be 4 digits unless it is > 31
+ ##### or MMYY is true
+ # M/YYYY, MM/YYYY
+ { length => [6,7], params => $MY, regex => qr{\A$MMYYYY\z} },
+ { length => [3..5], params => $MY, regex => qr{\A$MMYY\z},
+   postprocess => [sub {
+       my %args = @_;
+       if ( exists $args{args} )
+       {
+           my %original_args = @{$args{args}};
+           return 1 if ( $original_args{MMYY} );
+       }
+       return 1 if ( $args{parsed}{year} > 31 );
+       return 0;
+   }, \&_fix_year] },
+ ########################################################
  ##### Month/Day
  # M/D, M/DD, MM/D, MM/DD
- { length => [3..5], params => $MD, regex  => qr{\A$MMDD\z}, postprocess => sub { my %args = @_; $args{parsed}{year} = __PACKAGE__->base->year } },
+ { length => [3..5], params => $MD, regex => qr{\A$MMDD\z},
+   postprocess => sub { my %args = @_; $args{parsed}{year} = __PACKAGE__->base->year } },
 
  { length => [9..14], params => $MDHMS, regex => qr{\A$MMDD\s$HMS\z},
    postprocess => sub { my %args = @_; $args{parsed}{year} = __PACKAGE__->base->year } },
@@ -184,7 +205,21 @@ my $formats =
 
  ########################################################
  # YYYY HH:MM:SS
- { length => [13], params => $YHMS , regex => qr{\A$YEAR\s$HMS\z} } ,
+ { length => [13], params => $YHMS, regex => qr{\A$YEAR\s$HMS\z} } ,
+
+ ########################################################
+ # time first
+ # (5:30 12-10)
+ { length => [8..11], params => $HMMD, regex => qr{\A${HM}\s${MMDD}\z}, postprocess => \&_set_default_year },
+ # 5:30:25:05/1/1/65
+ # 12:30:25:05/10/10/65
+ { length => [17..20], params => $HMSNSMDY, regex => qr{\A${HMSNS}${DELIM}${MMDDYYYY}\z}, postprocess => \&_fix_year },
+ # 5:30:25 12101965
+ { length => [14..16], params => $HMSMDY, regex => qr{\A${HMS}${DELIM}${MON}${DAY}${YEAR}\z}, postprocess => \&_fix_year },
+ { length => [14..19], params => $HMSMDY, regex => qr{\A${HMS}${DELIM}${MMDDYYYY}\z}, postprocess => \&_fix_year },
+ # 5:30 pm 121065 => 2065-12-01T17:30:00
+ { length => [14,18], params => $HMAPMMDDYYYY, regex => qr{\A${HM}\s${AMPM}\s${MON}${DAY}${YEAR}},postprocess => [\&_fix_ampm, \&_fix_year] },
+ { length => [16,19], params => $HMAPMMDDYYYY, regex => qr{\A${HM}\s${AMPM}\s${MMDDYYYY}},postprocess => [\&_fix_ampm, \&_fix_year] },
 
  ########################################################
  ##### Alpha months
@@ -256,9 +291,11 @@ my $formats =
  # Dec 03 20:53:10 2009
  { length => [16..21], params => $MDHMSY , regex => qr{\AX(\d{1,2})X\s(\d{1,2})\s$HMS\s(\d{4})\z} } ,
  { length => [10..18], params => $HMMDY  , regex => qr{\A$HM\sX${MON}X\s$DAY\s$YEAR\z} },
- # 8:00 px Dec 10th => 8:00pm X12X n10n
+ # 8:00 pm Dec 10th => 8:00pm X12X n10n
  { length => [14..19]    , params => $HMAPMMDD , regex => qr{\A$HM\s?$AMPM\sX${MON}X\sn${DAY}n\z} ,
    postprocess => [sub { my %args = @_; $args{parsed}{year} = __PACKAGE__->base->year }, \&_fix_ampm] },
+ # 5:30 DeC 1
+ { length => [11], params => $HMMD, regex => qr{\A${HM}\sX${MON}X\s${DAY}\z}m, postprocess => \&_set_default_year },
 
  ########################################################
  ##### Bare Numbers
@@ -446,10 +483,19 @@ sub _fix_alpha
         base => __PACKAGE__->base,
     );
 
-    printf( "# before lang: %s\n", $date ) if $ENV{DFF_DEBUG};
-    ( $date , $p ) = $lang->_cleanup( $date , $p );
-    printf( "# after lang: %s\n", $date ) if $ENV{DFF_DEBUG};
+    my $stripped = $date;
+    $stripped =~ s{$DELIM|$HMSDELIM}{}gm;
 
+    if ( $stripped =~ m{(\D)} )
+    {
+        printf( "# before lang: %s\n", $date ) if $ENV{DFF_DEBUG};
+        ( $date , $p ) = $lang->_cleanup( $date , $p );
+        printf( "# after lang: %s\n", $date ) if $ENV{DFF_DEBUG};
+    }
+    else
+    {
+        printf( "# ignoring languages, no non numbers (%s)\n", $stripped ) if $ENV{DFF_DEBUG};
+    }
 
     $date =~ s{($DELIM)+}{$1}mxg;   # make multiple delimeters into one
     # remove any leading delimeters unless it is -infinity
@@ -494,6 +540,16 @@ sub _parse_timezone
         }
     }
 
+    # search for GMT inside the string
+    # must be surrounded by spaces
+    # 5:30 pm GMT 121065
+    if ( my ( $tz ) = $date =~ m{\s(GMT)\s}mx )
+    {
+        $date =~ s{\Q$tz\E}{};
+        $p->{time_zone} = 'UTC';
+        return ( $date , $p );
+    }
+
     # remove any trailing 'Z' => UTC
     if ( $date =~ m{Z\z}mx )
     {
@@ -667,6 +723,8 @@ sub _pick_year
 
 __END__
 
+=encoding utf-8
+
 =head1 NAME
 
 DateTime::Format::Flexible - DateTime::Format::Flexible - Flexibly parse strings and turn them into DateTime objects.
@@ -674,17 +732,19 @@ DateTime::Format::Flexible - DateTime::Format::Flexible - Flexibly parse strings
 =head1 SYNOPSIS
 
   use DateTime::Format::Flexible;
-  my $dt = DateTime::Format::Flexible->parse_datetime( 'January 8, 1999' );
+  my $dt = DateTime::Format::Flexible->parse_datetime(
+      'January 8, 1999'
+  );
   # $dt = a DateTime object set at 1999-01-08T00:00:00
 
 =head1 DESCRIPTION
 
-If you have ever had to use a program that made you type in the date a certain
-way and thought "Why can't the computer just figure out what date I wanted?",
-this module is for you.
+If you have ever had to use a program that made you type in the
+date a certain way and thought "Why can't the computer just figure
+out what date I wanted?", this module is for you.
 
-F<DateTime::Format::Flexible> attempts to take any string you give it and parse
-it into a DateTime object.
+F<DateTime::Format::Flexible> attempts to take any string you give
+it and parse it into a DateTime object.
 
 =head1 USAGE
 
@@ -692,7 +752,8 @@ This module uses F<DateTime::Format::Builder> under the covers.
 
 =head2 parse_datetime
 
-Give it a string and it attempts to parse it and return a DateTime object.
+Give it a string and it attempts to parse it and return a DateTime
+object.
 
 If it cannot it will throw an exception.
 
@@ -703,7 +764,7 @@ If it cannot it will throw an exception.
      strip    => [qr{\.\z}],                  # optional, remove a trailing period
      tz_map   => {EDT => 'America/New_York'}, # optional, map the EDT timezone to America/New_York
      lang     => ['es'],                      # optional, only parse using spanish
-     european => 1                            # optional, catch some cases of DD-MM-YY
+     european => 1,                           # optional, catch some cases of DD-MM-YY
  );
 
 =over 4
@@ -718,7 +779,7 @@ example:
  my $base_dt = DateTime->new( year => 2005, month => 2, day => 1 );
  my $dt = DateTime::Format::Flexible->parse_datetime(
     '18 Mar',
-     base => $base_dt
+     base => $base_dt,
  );
  # $dt is now 2005-03-18T00:00:00
 
@@ -730,76 +791,108 @@ You can pass multiple regexes in an arrayref.
 example:
 
  my $dt = DateTime::Format::Flexible->parse_datetime(
-     '2011-04-26 00:00:00 (registry time)' ,
-     strip => [qr{\(registry time\)\z}] ,
+     '2011-04-26 00:00:00 (registry time)',
+     strip => [qr{\(registry time\)\z}],
  );
  # $dt is now 2011-04-26T00:00:00
 
-This is helpful if you have a load of dates you want to normalize and you know
-of some weird formatting beforehand.
+This is helpful if you have a load of dates you want to normalize
+and you know of some weird formatting beforehand.
 
 =item * C<tz_map> (optional)
 
-map a given timezone to another recognized timezone
+Map a given timezone to another recognized timezone
 Values are given as a hashref.
 
 example:
 
  my $dt = DateTime::Format::Flexible->parse_datetime(
-     '25-Jun-2009 EDT' ,
-     tz_map => {EDT => 'America/New_York'}
+     '25-Jun-2009 EDT',
+     tz_map => {EDT => 'America/New_York'},
  );
  # $dt is now 2009-06-25T00:00:00 with a timezone of America/New_York
 
-This is helpful if you have a load of dates that have timezones that are not
-recognized by F<DateTime::Timezone>.
+This is helpful if you have a load of dates that have timezones that
+are not recognized by F<DateTime::Timezone>.
 
 =item * C<lang> (optional)
 
 Specify the language map plugins to use.
 
-When DateTime::Format::Flexible parses a date with a string in it, it will
-search for a way to convert that string to a number.  By default it will
-search through all the language plugins to search for a match.
+When DateTime::Format::Flexible parses a date with a string in it,
+it will search for a way to convert that string to a number.  By
+default it will search through all the language plugins to search
+for a match.
 
-Setting this lets you limit the scope of the search
+NOTE: as of 0.22, it will only do this search if it detects a string
+in the given date.
+
+Setting C<lang> this lets you limit the scope of the search.
 
 example:
 
  my $dt = DateTime::Format::Flexible->parse_datetime(
-     'Wed, Jun 10, 2009' ,
-     lang => ['en']
+     'Wed, Jun 10, 2009',
+     lang => ['en'],
  );
  # $dt is now 2009-06-10T00:00:00
 
-Currently supported languages are english (en) and spanish (es).
-Contributions, corrections, requests and examples are VERY welcome.
-See the F<DateTime::Format::Flexible::lang::en> and
-F<DateTime::Dormat::Flexible::lang::es> for examples of the plugins.
+Currently supported languages are english (en), spanish (es) and
+german (de). Contributions, corrections, requests and examples
+are VERY welcome.
+
+See the F<DateTime::Format::Flexible::lang::en>,
+F<DateTime::Format::Flexible::lang::es>, and
+F<DateTime::Format::Flexible::lang::de>
+for examples of the plugins.
 
 =item * C<european> (optional)
 
-If european is set to a true value, an attempt will be made to parse as a
-DD-MM-YYYY date instead of the default MM-DD-YYYY.  There is a chance
-that this will not do the right thing due to ambiguity.
+If european is set to a true value, an attempt will be made to parse
+as a DD-MM-YYYY date instead of the default MM-DD-YYYY.  There is a
+chance that this will not do the right thing due to ambiguity.
 
 example:
 
  my $dt = DateTime::Format::Flexible->parse_datetime(
-     '16/06/2010' , european => 1 ,
+     '16/06/2010' , european => 1,
  );
  # $dt is now 2010-06-16T00:00:00
 
+=item * C<MMYY> (optional)
+
+By default, this module parse 12/10 as December 10th of the current
+year (MM/DD).
+
+If you want it to parse this as MM/YY instead, you can enable the
+C<MMYY> option.
+
+example:
+
+ my $dt = DateTime::Format::Flexible->parse_datetime('12/10');
+ # $dt is now [current year]-12-10T00:00:00
+
+ my $dt = DateTime::Format::Flexible->parse_datetime(
+     '12/10', MMYY => 1,
+ );
+ # $dt is now 2010-12-01T00:00:00
+
+This is useful if you know you are going to be parsing a credit card
+expiration date.
+
 =back
 
 =head2 base
 
-gets/sets the base DateTime for incomplete dates.  Requires a valid DateTime
-object as an argument when setting.
+gets/sets the base DateTime for incomplete dates.  Requires a valid
+DateTime object as an argument when setting.  This defaults to
+DateTime->now.
 
 example:
 
- DateTime::Format::Flexible->base( DateTime->new( year => 2009, month => 6, day => 22 ) );
+ DateTime::Format::Flexible->base( DateTime->new(
+     year => 2009, month => 6, day => 22
+ ));
  my $dt = DateTime::Format::Flexible->parse_datetime( '23:59' );
  # $dt is now 2009-06-22T23:59:00
 
@@ -815,54 +908,55 @@ A small list of supported formats:
 
 =item YYYYMMDDTHHMMSS
 
-=item  YYYYMMDDTHHMM
+=item YYYYMMDDTHHMM
 
-=item  YYYYMMDDTHH
+=item YYYYMMDDTHH
 
-=item  YYYYMMDD
+=item YYYYMMDD
 
-=item  YYYYMM
+=item YYYYMM
 
-=item  MM-DD-YYYY
+=item MM-DD-YYYY
 
-=item  MM-D-YYYY
+=item MM-D-YYYY
 
-=item  MM-DD-YY
+=item MM-DD-YY
 
-=item  M-DD-YY
+=item M-DD-YY
 
-=item  YYYY/DD/MM
+=item YYYY/DD/MM
 
-=item  YYYY/M/DD
+=item YYYY/M/DD
 
-=item  YYYY/MM/D
+=item YYYY/MM/D
 
-=item  M-D
+=item M-D
 
-=item  MM-D
+=item MM-D
 
-=item  M-D-Y
+=item M-D-Y
 
-=item  Month D, YYYY
+=item Month D, YYYY
 
-=item  Mon D, YYYY
+=item Mon D, YYYY
 
-=item  Mon D, YYYY HH:MM:SS
+=item Mon D, YYYY HH:MM:SS
 
-=item  ...
+=item ... thousands more
 
 =back
 
-there are 9000+ variations that are detected correctly in the test files
-(see t/data/* for most of them).  If you can think of any that I do not
-cover, please let me know.
+there are 9000+ variations that are detected correctly in the test
+files (see t/data/* for most of them).  If you can think of any that
+I do not cover, please let me know.
 
 =head1 NOTES
 
 As of version 0.11 you will get a DateTime::Infinite::Future object
-if the passed in date is 'infinity' and a DateTime::Infinite::Past object
-if the passed in date is '-infinity'.  If you are expecting these types of
-strings, you might want to check for 'is_infinite()' from the object returned.
+if the passed in date is 'infinity' and a DateTime::Infinite::Past
+object if the passed in date is '-infinity'.  If you are expecting
+these types of strings, you might want to check for
+'is_infinite()' from the object returned.
 
 example:
 
@@ -872,23 +966,10 @@ example:
       # you have a Infinite object.
  }
 
-The DateTime website http://datetime.perl.org/?Modules as of february 2010
-lists this module under 'Confusing' and recommends the use of
-F<DateTime::Format::Natural>.
-
-Unfortunately I do not agree.  F<DateTime::Format::Natural> fails
-more than 2000 of my parsing tests.  F<DateTime::Format::Flexible> supports
-different types of date/time strings than F<DateTime::Format::Natural>.
-I think there is utility in that can be found in both of them.
-
-The whole goal of F<DateTime::Format::Flexible> is to accept just about
-any crazy date/time string that a user might care to enter.
-F<DateTime::Format::Natural> seems to be a little stricter in what it can
-parse.
-
 =head1 BUGS/LIMITATIONS
 
-You cannot use a 1 or 2 digit year as the first field unless the year is > 31:
+You cannot use a 1 or 2 digit year as the first field unless the
+year is > 31:
 
  YY-MM-DD # not supported if YY is <= 31
  Y-MM-DD  # not supported
@@ -901,7 +982,7 @@ Tom Heady <cpan@punch.net>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2007-2011 Tom Heady.
+Copyright 2007-2012 Tom Heady.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of either:
@@ -912,7 +993,7 @@ modify it under the terms of either:
     Software Foundation; either version 1, or (at your option) any
     later version, or
 
-=item * the Artistic License version 2.0.
+=item * the Artistic License.
 
 =back
 
